// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file LGPL included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.41 $

#include "hk_kdesimplereport.h"
#include "hk_kdereportsection.h"
#include "hk_kdereportproperty.h"
#include "hk_kdereport.h"
#include "hk_kdereportdata.h"
#include <hk_reportsectionpair.h>
#include <qlayout.h>
#include <hk_kdeformfocus.h>
#include <hk_datasource.h>
#include <qkeycode.h>
#include <qpopupmenu.h>
#include <qfocusdata.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qpopupmenu.h>
#include <klocale.h>
#include <kaction.h>
#include <qdragobject.h>
#include <qclipboard.h>
#include <qcolor.h>

#ifdef HAVE_SSTREAM
#include <sstream>
#else
#include <strstream.h>
#endif

hk_kdesimplereport::hk_kdesimplereport (QWidget* parent,  const char* name, WFlags fl )
: QWidget( parent, name, fl ) ,hk_report()
{
#ifdef HK_DEBUG
//wanna_debug(true);
    hkdebug("hk_kdesimplereport::hk_kdesimplereport");
#endif
    p_popup=NULL;
    p_deleteaction=NULL;
    p_copyaction=NULL;
    p_pasteaction=NULL;
    p_cutaction=NULL;
    p_resizeaction=NULL;
    p_minwidthaction=NULL;
    p_maxwidthaction=NULL;
    p_minheightaction=NULL;
    p_maxheightaction=NULL;
    p_minsizeaction=NULL;
    p_maxsizeaction=NULL;
    p_alignaction=NULL;
    p_alignleftaction=NULL;
    p_alignrightaction=NULL;
    p_aligntopaction=NULL;
    p_alignbottomaction=NULL;
    setGeometry(0,0,800,600);
    p_sourcevanishflag=false;
    p_focus = NULL;
    p_property=NULL;
    p_focussection=NULL;
    p_progressdialog=NULL;
    set_reporttype("Postscript",false);
    create_propertyeditor();
    p_property->show();
    set_propertyobject(this);
    p_reportlayout=NULL;
    p_autoclose=true;
    p_createfield=false;
    p_kdereport=NULL;
    init_report();
    set_sections();
    setFocusPolicy(StrongFocus);
    setFocus();
    reset_has_changed();
    connect (QApplication::clipboard(), SIGNAL(dataChanged()),this, SLOT(enable_actions()));
    hk_colour c=foregroundcolour();
    QColor newcolour(c.red(),c.green(),c.blue());
    setPaletteForegroundColor(newcolour);
    c=backgroundcolour();
    newcolour.setRgb(c.red(),c.green(),c.blue());
    setPaletteBackgroundColor(newcolour);
}


hk_kdesimplereport::~hk_kdesimplereport(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::~hk_kdesimplereport");
#endif
    delete p_deleteaction;
    delete p_copyaction;
    delete p_pasteaction;
    delete p_cutaction;

}


void hk_kdesimplereport::set_kdereport(hk_kdereport* k)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::set_kdereport");
#endif
    p_kdereport=k;
    if (!p_kdereport)return;
    p_deleteaction=new KAction(i18n("&Delete"),Key_Delete,this,SLOT(delete_widgets()),p_kdereport->actionCollection(),"deleteclicked");
    p_copyaction=new KAction(i18n("&Copy"),CTRL+Key_C,this,SLOT(copy_widgets()),p_kdereport->actionCollection(),"copyclicked");
    p_pasteaction=new KAction(i18n("&Paste"),CTRL+Key_V,this,SLOT(paste_widgets()),p_kdereport->actionCollection(),"pasteclicked");
    p_cutaction=new KAction(i18n("Cu&t"),CTRL+Key_X,this,SLOT(cut_widgets()),p_kdereport->actionCollection(),"cutclicked");
    p_resizeaction=new KActionMenu(i18n("Adjust &size"),0,p_kdereport->actionCollection(),"size");
    p_minwidthaction=new KAction(i18n("Minimum width"),0,0,this,SLOT(adjust_minw()),p_kdereport->actionCollection(),"minwidth");
    p_maxwidthaction=new KAction(i18n("Maximum width"),0,0,this,SLOT(adjust_maxw()),p_kdereport->actionCollection(),"maxwidth");
    p_minheightaction=new KAction(i18n("Minimum height"),0,0,this,SLOT(adjust_minh()),p_kdereport->actionCollection(),"minheight");
    p_maxheightaction=new KAction(i18n("Maximum height"),0,0,this,SLOT(adjust_maxh()),p_kdereport->actionCollection(),"maxheight");
    p_minsizeaction=new KAction(i18n("Minimum size"),0,0,this,SLOT(adjust_mins()),p_kdereport->actionCollection(),"minsize");
    p_maxsizeaction=new KAction(i18n("Maximum size"),0,0,this,SLOT(adjust_maxs()),p_kdereport->actionCollection(),"maxsize");
    p_resizeaction->insert(p_minwidthaction);
    p_resizeaction->insert(p_maxwidthaction);
    p_resizeaction->insert(p_minheightaction);
    p_resizeaction->insert(p_maxheightaction);
    p_resizeaction->insert(p_minsizeaction);
    p_resizeaction->insert(p_maxsizeaction);

    p_alignaction=new KActionMenu(i18n("&Align"),0,p_kdereport->actionCollection(),"align");
    p_alignleftaction=new KAction(i18n("&Left"),0,0,this,SLOT(align_left()),p_kdereport->actionCollection(),"alignleft");
    p_alignrightaction=new KAction(i18n("&Right"),0,0,this,SLOT(align_right()),p_kdereport->actionCollection(),"alignright");
    p_aligntopaction=new KAction(i18n("&Top"),0,0,this,SLOT(align_top()),p_kdereport->actionCollection(),"aligntop");
    p_alignbottomaction=new KAction(i18n("&Bottom"),0,0,this,SLOT(align_bottom()),p_kdereport->actionCollection(),"alignbottom");
    p_alignaction->insert(p_alignleftaction);
    p_alignaction->insert(p_alignrightaction);
    p_alignaction->insert(p_aligntopaction);
    p_alignaction->insert(p_alignbottomaction);

    enable_actions();
}


hk_reportsection* hk_kdesimplereport::widget_specific_new_section(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::widget_specific_new_section");
#endif
    hk_reportsection* s=new hk_kdereportsection(this);
    return s;
}


void hk_kdesimplereport::create_propertyeditor(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::create_propertyeditor");
#endif
    if (p_property==NULL)
    {
        p_property=new hk_kdereportproperty(this,0,false);
    }

}


bool hk_kdesimplereport::eventFilter(QObject* object,QEvent* event)
{
#ifdef HK_DEBUG
//hkdebug("hk_kdesimplereport::eventFilter");
#endif
    if (   mode()!=design
        ||!object->isWidgetType()
        )
        return QWidget::eventFilter(object,event);
    hk_visible* v = dynamic_cast <hk_visible*>(object);
    hk_visible* p = dynamic_cast <hk_visible*> ( ((QWidget*) object)->parentWidget() );
    if ((v==NULL&& p==NULL)|| dynamic_cast <hk_marker*>(object)!=NULL||!is_reportobject((QWidget*) object))
    {
        return QWidget::eventFilter(object,event);
    }
    return reporteventFilter(object,event);
}


bool hk_kdesimplereport::reporteventFilter(QObject* object,QEvent* event)
{
#ifdef HK_DEBUG
//hkdebug("hk_kdesimpleform::formeventFilter");
#endif

    QWidget* widget= (QWidget*) object;

    switch (event->type())
    {
        case QEvent::KeyPress        :
        {
            keyPressEvent((QKeyEvent*)event);
            break;
        };
        case QEvent::KeyRelease      :
        {
            keyReleaseEvent((QKeyEvent*)event);
            break;
        };
        case QEvent::MouseButtonPress    :
        {

            if (object==this)
            {
                if (((QMouseEvent*)event)->button() !=RightButton)
                {
                    mousePressEvent((QMouseEvent*) event);

                    return true;
                }
            }
            else
            if (((QMouseEvent*)event)->button() ==LeftButton)
            {                                     //select object
                                                  // i.e. a menu
                if ( dynamic_cast <QPopupMenu*>(object))
                    return QWidget::eventFilter(object,event);
                if ( dynamic_cast <hk_kdereportsection*>(object))
                    return QWidget::eventFilter(object,event);

                hk_kdereportsection* s=NULL;
                hk_kdereportdata* d=dynamic_cast<hk_kdereportdata*>(object);
                if (d!=NULL) s=d->section();
                set_focus(widget,s);

                if (widget!=NULL&&p_focus!=NULL)p_focus->show();
// the following allows the label widget of a hk_kdereportsection to
// activate the section on a mousepressevent
                if (!s && widget) return QWidget::eventFilter(object,event);
                p_originalposition=mapFromGlobal(((QMouseEvent*)event)->globalPos());
                return true;
            }
            else
            if (((QMouseEvent*)event)->button() ==RightButton)
            {
                mousemenu((QMouseEvent*)event);
                return true;
            }
            break;
        };
        case QEvent::MouseMove       :
        {
            if (p_focus!=NULL)
                if (
                (((QMouseEvent*)event)->state() & LeftButton ) == LeftButton&&
                widget==p_focus->widget()
                )
            {
//move object
                QPoint newpos=mapFromGlobal(((QMouseEvent*)event)->globalPos());
                QPoint difference=p_originalposition-newpos;
                p_originalposition=newpos;
                if ( widget->x()-difference.x()<0) difference.setX(widget->x());
                if ( widget->y()-difference.y()<0) difference.setY(widget->y());
                if (p_focussection!=NULL)
                {
//cout <<"wx: "<<widget->x()<<" ww: "<<widget->width()<<" md: "<<p_focussection->max_displaywidth()<<endl;
                    if ( widget->x()+widget->width()-difference.x()>(signed)p_focussection->max_displaywidth())
                        difference.setX(-(p_focussection->max_displaywidth()-widget->x()-widget->width()));

                    if (p_focussection==page_header()||p_focussection==page_footer())
                    {
                        if (p_focussection==page_header())
                        {
                            if (sizetype()==hk_presentation::relative)
			    {
			    if ( widget->y()+widget->height()-difference.y()>(signed)relativ2vertical(border_top())*32.0/100+0.5)
                                                  //limits size of header section
                                difference.setY((int)-(relativ2vertical(border_top())*32.0/100-widget->y()-widget->height()+0.5));
                             }
			   else
			   {
			    if ( widget->y()+widget->height()-difference.y()>(signed)border_top()*32.0/100+0.5)
                                                  //limits size of header section
                                difference.setY((int)-(border_top()*32.0/100-widget->y()-widget->height()+0.5));
			   
			   }  
			}
                        else                      //page_FOOTER
                        {
                            if (sizetype()==hk_presentation::relative)
			    {
                            if ( widget->y()+widget->height()-difference.y()>(signed)relativ2vertical(border_bottom())*32.0/100+0.5)
                                                  //limits size of footer section
                                difference.setY((int)-(relativ2vertical(border_bottom())*32.0/100-widget->y()-widget->height()+0.5));
                            }
			    else
			    {
                            if ( widget->y()+widget->height()-difference.y()>(signed)border_bottom()*32.0/100+0.5)
                                                  //limits size of footer section
                                difference.setY((int)-(border_bottom()*32.0/100-widget->y()-widget->height()+0.5));
                            }
			}
                    }
                    else
                    {
                        if ( widget->y()+widget->height()-difference.y()>(signed)p_focussection->max_displayheight())
                            difference.setY(-(p_focussection->max_displayheight()-widget->y()-widget->height()));
                    }
                } else cout <<"p_focussection==NULL!!!"<<endl;
                widget->move(widget->x()-difference.x(),widget->y()-difference.y());
                if (widget->parentWidget()!=NULL)
                {
                    int b=widget->y()+widget->height();
                    if (widget->parentWidget()->height()<b)
                    {
                        if (p_focussection!=NULL)
                            if (p_focussection->max_displayheight()<(unsigned int)b)
                                widget->parentWidget()->setFixedHeight(b);cout <<"hk_kdesimplereport setFixedheight"<<endl;
                    }
                }

                p_focus->set_positions();
                p_property->focus_resized();
            }
            break;
        };

        case QEvent::MouseButtonDblClick : return true;
        default              :;

    }

    return QWidget::eventFilter(object,event);
}


void hk_kdesimplereport::set_focus(QWidget*f,hk_kdereportsection* s)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::set_focus");
#endif
    if (f==NULL || f==this ||s==NULL)
    {
        clearfocus();
        return;

    }

    if (s==f)
    {
//set sectionproperty
        set_actualsection(s);
        clearfocus();
        set_propertyobject(s);
        return;

    }
    set_actualsection(s);
    if (p_focus==NULL) p_focus=s->new_focus();
    if (p_controlbutton==true)
    {

        if (p_focus->widget()==f ) return;
        list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
        while (it!=p_multiplefocus.end())         //check if widget is already set
        {
            hk_kdeformfocus* w=(*it);
            it++;
            if (w->widget()==f) return;
        }

    }

    if (p_controlbutton==true&& p_focus->widget()!=NULL &&p_focus->widget()!=this)
    {
        p_multiplefocus.insert(p_multiplefocus.end(),p_focus);
        hk_kdeformfocus* pf=p_focus;
        p_focus=NULL;
        pf->repaint();
    }
    else clearmultiplefocus();
    if (p_focus!=NULL) delete p_focus;
    p_focus = s->new_focus();
    p_focus->set_widget(f);
    set_propertyobject(f);
    enable_actions();
}


void hk_kdesimplereport::set_actualsection(hk_kdereportsection* s)
{
    p_focussection=s;

}


void hk_kdesimplereport::set_propertyobject(QWidget* f)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::set_propertyobject");
#endif
    hk_visible* v= dynamic_cast<hk_visible*>(f);
    if (p_property!=NULL)
    {
        if (p_focus!=NULL)QObject::connect(p_focus,SIGNAL(size_changed()),p_property,SLOT(focus_resized()));
        if (v==NULL) v=this;
        p_property->set_object(v);
    }

}


void hk_kdesimplereport::clearfocus(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::clearfocus");
#endif
    if (p_focus!=NULL)
    {
        delete p_focus;
        p_focus=NULL;

    }
    p_controlbutton=false;
    clearmultiplefocus();
    enable_actions();
}


void hk_kdesimplereport::clearmultiplefocus(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::clearmultiplefocus");
#endif
    list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
    while (it!=p_multiplefocus.end())
    {
        hk_kdeformfocus* f=(*it);
        it++;
        if (f!=p_focus) delete f;
    }
    p_multiplefocus.erase(p_multiplefocus.begin(),p_multiplefocus.end());
    enable_actions();
}


void hk_kdesimplereport::before_source_vanishes(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::before_source_vanishes");
#endif
    hk_report::before_source_vanishes();

    if (p_autoclose && !p_sourcevanishflag)
    {
        if (parentWidget()!=NULL)
        {
            emit closereport();
        }
    }
    p_sourcevanishflag=true;

}


void hk_kdesimplereport::database_has_vanished(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::database_has_vanished");
#endif
    if (p_autoclose && !p_sourcevanishflag)
    {
        if (parentWidget()!=NULL)
        {
            emit closereport();
        }
    }
    p_sourcevanishflag=true;
}


void hk_kdesimplereport::set_field2create(bool c)
{
    p_createfield=c;

}


bool hk_kdesimplereport::field2create(void)
{
    return p_createfield;
}


bool hk_kdesimplereport::widget_specific_before_modechanges(enum_mode m)
{
    if (m==view)
    {

        return   execute();

    }
    else return true;

}


void hk_kdesimplereport::widget_specific_modechanges(enum_mode s)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::widget_specific_modechanges");
#endif
    list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
    if(s==design)
    {
        setFocusPolicy(StrongFocus);
        setFocus();
        qApp->installEventFilter(this);
        if (p_property==NULL)
        {
            create_propertyeditor();
        }
        if (p_focus!=NULL)
        {
            if (p_focus->widget()!=NULL)
                p_property->set_object( dynamic_cast <hk_visible*> (p_focus->widget()));
            else p_property->set_object(this);
        }
        while (it!=p_multiplefocus.end())
        {

            hk_kdeformfocus* f=(*it);
            f->show();
            it++;
        }
        if (p_focus!=NULL)p_focus->show();
        p_property->show();

    }
    else                                          // s!=design
    {
        qApp->removeEventFilter(this);
        while (it!=p_multiplefocus.end())
        {
            hk_kdeformfocus* f=(*it);
            f->hide();
            it++;
        }
        if (p_focus!=NULL) p_focus->hide();
        if(p_property!=NULL) p_property->hide();
        setFocusPolicy(ClickFocus);
    }
    enable_actions();
}


void hk_kdesimplereport::keyPressEvent ( QKeyEvent * event )
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::keyPressEvent");
#endif
    switch(event->key())
    {
        case Key_Control:  p_controlbutton=true;
        break;
        case Key_Delete:   delete_widgets();
        break;
/*   case Key_Left:
            if (p_focus &&p_focus->widget())
            {
              hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
              if (widget)
              {
               unsigned int diff=10;
               if (widget->x()>=diff) widget->set_x(widget->x()-diff);
              }

            }
break;
case Key_Right:
if (p_focus &&p_focus->widget())
{
hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
if (widget)
{
unsigned int diff=10;
if (widget->x()+widget->width()+diff<=10000) widget->set_x(widget->x()+diff);
}

}
break;
case Key_Up:
if (p_focus &&p_focus->widget())
{
hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
if (widget)
{
unsigned int diff=10;
if (widget->y()>=diff) widget->set_y(widget->y()-diff);
}

}
break;
case Key_Down:
if (p_focus &&p_focus->widget())
{
hk_visible* widget=dynamic_cast<hk_visible*>(p_focus->widget());
if (widget &&p_focussection)
{
unsigned int diff=10;
if (p_focussection==page_header()||p_focussection==page_footer())
{
if (p_focussection==page_header())
{
if ( widget->y()+widget->height()+diff>=border_top())
diff=border_top()-widget->y()-widget->height();//limits size of header section
}
else//page_FOOTER
{
if ( widget->y()+widget->height()+diff>border_bottom())
diff=border_bottom()-widget->y()-widget->height();//limits size of footer section
}
}
if (widget->y()+widget->height()+diff<=10000) widget->set_y(widget->y()+diff);
}
}
break;
        */
        default    :     ;                        
    }
    QWidget::keyPressEvent(event);

}


void hk_kdesimplereport::keyReleaseEvent ( QKeyEvent * event)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::keyReleaseEvent");
#endif
    if (event->key()==Key_Control)  p_controlbutton=false;
    QWidget::keyReleaseEvent(event);

}


void hk_kdesimplereport::delete_widgets(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::delete_widgets");
#endif
    QWidget* w=NULL;
//QWidget* del=NULL;
    list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
    while (it!=p_multiplefocus.end())
    {
        hk_kdeformfocus* f=(*it);
        it++;
        w=f->widget();
//del=w;
        f->set_widget(NULL);
//     remove_visible(dynamic_cast <hk_visible*>(del));
        delete w;
    }
    if (p_focus!=NULL)
    {
        w=p_focus->widget();
//del=w;
        p_focus->set_widget(NULL);
    }
//  remove_visible(dynamic_cast <hk_visible*>(del));
    if (p_property!=NULL) p_property->set_object(this);
    delete w;
    clearfocus();
    set_has_changed();
}


hk_kdeformfocus* hk_kdesimplereport::formfocus(void)
{
    return p_focus;
}


void hk_kdesimplereport::field_created(void)
{
    emit signalfield_created();
}


void hk_kdesimplereport::set_focus2property(void)
{

    clearfocus();
    set_propertyobject(this);

}


void hk_kdesimplereport::set_sections(void)
{

    delete p_reportlayout;
    p_reportlayout=new QVBoxLayout(this);
    p_reportlayout->setResizeMode(QLayout::Fixed);
    p_reportlayout->setSpacing(0);
    p_reportlayout->setMargin(0);

    p_reportlayout->addWidget((dynamic_cast<hk_kdereportsection*>(page_header()))->header());
    p_reportlayout->addWidget(dynamic_cast<hk_kdereportsection*>(page_header()));
    p_reportlayout->addWidget((dynamic_cast<hk_kdereportsection*>(report_header()))->header());
    p_reportlayout->addWidget(dynamic_cast<hk_kdereportsection*>(report_header()));
    vector<hk_reportsectionpair*>* s=sectionpairs();
    vector<hk_reportsectionpair*>::iterator it=s->begin();
    while (it!=s->end())
    {
        hk_kdereportsection* s=dynamic_cast<hk_kdereportsection*>((*it)->headersection());
        if (s!=NULL)
        {
            s->set_displaystring(i18n("Sectionheader"));
            p_reportlayout->addWidget(s->header());
            p_reportlayout->addWidget(s);
            s->show();
            s->header()->show();
        }
        it++;
    }

    hk_kdereportsection* sec=dynamic_cast<hk_kdereportsection*>(datasection());

    if (sec!=NULL)
    {
        p_reportlayout->addWidget(sec->header());
        p_reportlayout->addWidget(sec);
        sec->set_displaystring(i18n("Datasection"));
        sec->show();

    }

    int i=s->size();
    if (i>0)
    {
        while (i>0)
        {
            hk_kdereportsection* sec=dynamic_cast<hk_kdereportsection*>((*s)[i-1]->footersection());
            p_reportlayout->addWidget(sec->header());
            p_reportlayout->addWidget(sec);
            sec->set_displaystring(i18n("Sectionfooter"));
            sec->header()->show();
            sec->show();
            i--;
        }
    }

    p_reportlayout->addWidget((dynamic_cast<hk_kdereportsection*>(report_footer()))->header());
    p_reportlayout->addWidget(dynamic_cast<hk_kdereportsection*>(report_footer()));
    p_reportlayout->addWidget((dynamic_cast<hk_kdereportsection*>(page_footer()))->header());
    p_reportlayout->addWidget(dynamic_cast<hk_kdereportsection*>(page_footer()));

    hk_kdereportsection* k=static_cast <hk_kdereportsection*>(page_header());
    k->set_displaystring(i18n("Pageheader"));
    k->show();
    k=static_cast <hk_kdereportsection*>(page_footer());
    k->set_displaystring(i18n("Pagefooter"));
    k->show();
    k=static_cast <hk_kdereportsection*>(report_header());
    k->set_displaystring(i18n("Reportheader"));
    k->show();
    k=static_cast <hk_kdereportsection*>(report_footer());
    k->set_displaystring(i18n("Reportfooter"));
    k->show();
}


bool hk_kdesimplereport::load_report(const hk_string& name)
{
    bool res=hk_report::load_report(name);
    set_sections();
    return res;
}


void hk_kdesimplereport::widget_specific_after_loadreport()
{
    clearfocus();
    p_property->set_object(this);
    p_property->datasource_selected();
// qApp->processEvents();

}


void    hk_kdesimplereport::widget_specific_before_execute(void)
{
    p_cancelprinting=false;
    p_progressdialog= new  QProgressDialog();
    p_progressdialog->setCaption(i18n("Generating report..."));
    p_progressdialog->setTotalSteps(datasource()->max_rows());
    connect(p_progressdialog,SIGNAL(cancelled()),this,SLOT(printing_cancelled()));
    p_progressdialog->show();
    qApp->processEvents();
}


void    hk_kdesimplereport::widget_specific_after_execute(void)
{
    delete p_progressdialog;
    p_progressdialog=NULL;
}


bool hk_kdesimplereport::widget_specific_after_new_page(void)
{
    if (p_progressdialog!=NULL)
    {
        p_progressdialog->setProgress(datasource()->row_position());
    }
    qApp->processEvents();

    return !p_cancelprinting;
}


void hk_kdesimplereport::printing_cancelled(void)
{
    p_cancelprinting=true;

}


void hk_kdesimplereport::mousemenu(QMouseEvent* m)
{

    if (p_popup!=NULL) return;
    p_popup=new QPopupMenu(this);
    if (p_multiplefocus.size()>0)
    {

        QPopupMenu* p_adjustsize =new QPopupMenu(this);
        QPopupMenu* p_align =new QPopupMenu(this);
        p_popup->insertItem( i18n( "Adjust &size" ),p_adjustsize,1 );
        p_popup->insertItem( i18n( "&Align" ),p_align,2 );

        p_adjustsize->insertItem( i18n( "Minimum width" ),3 );
        p_adjustsize->insertItem( i18n( "Maximum width" ),4 );
        p_adjustsize->insertItem( i18n( "Minimum height" ),5 );
        p_adjustsize->insertItem( i18n( "Maximum height" ),6 );
        p_adjustsize->insertItem( i18n( "Minimum size" ),7 );
        p_adjustsize->insertItem( i18n( "Maximum size" ),8 );

        p_align->insertItem( i18n( "&Left" ),9 );
        p_align->insertItem( i18n( "&Right" ),10 );
        p_align->insertItem( i18n( "&Top" ),11 );
        p_align->insertItem( i18n( "&Bottom" ),12);
    }
    QPopupMenu* p_settoall =new QPopupMenu(this);
    p_popup->insertItem(i18n("Set to all"),p_settoall,18);
    p_settoall->insertItem(i18n("Font"),19);
    p_settoall->insertItem(i18n("Foregroundcolour"),20);
    p_settoall->insertItem(i18n("Backgroundcolour"),21);

    if (p_focus!=NULL)
        if (p_focus->widget()!=NULL)
    {
        p_popup->insertItem( i18n( "&Delete" ),13 );
        p_popup->insertSeparator();
        p_popup->insertItem(i18n("Into background"),16);
        p_popup->insertItem(i18n("Into foreground"),17);
        p_popup->insertSeparator();
    }
    p_popup->insertItem( i18n( "Show &property editor" ),14 );
    p_popup->insertItem( i18n( "&Save report" ),15 );

    int r = p_popup->exec( m->globalPos() );
    switch (r)
    {
        case 3:       adjust_widgets(sminw);
        break;
        case 4:       adjust_widgets(smaxw);
        break;
        case 5:       adjust_widgets(sminh);
        break;
        case 6:       adjust_widgets(smaxh);
        break;
        case 7:       adjust_widgets(smins);
        break;
        case 8:       adjust_widgets(smaxs);;
        break;
        case 9:       adjust_widgets(al);
        break;
        case 10:      adjust_widgets(ar);
        break;
        case 11:      adjust_widgets(at);
        break;
        case 12:      adjust_widgets(ab);
        break;
        case 13:      delete_widgets();
        break;
        case 14:
        {
            if (p_property==NULL)
            {
                p_property=new hk_kdereportproperty(this,0,false);
            }
            if (p_property!=NULL)  p_property->show();
            break;
        }
        case 15:
        {
            save_report("",false);
            break;
        }
        case 16:
        {
            hk_visible* v=dynamic_cast<hk_visible*>(p_focus->widget());
            if (v)
            {
                v->lower_widget();
                p_focus->widget()->lower();
                p_focus->set_widget(p_focus->widget());
            }
            if (p_multiplefocus.size()>0)
            {
                list<hk_kdeformfocus*>::reverse_iterator it=p_multiplefocus.rbegin();
                while(it!=p_multiplefocus.rend())
                {

                    v=dynamic_cast<hk_visible*>((*it)->widget());
                    if (v)
                    {
                        v->lower_widget();
                        (*it)->widget()->lower();
                        ++it;
                    }
                }
            }

            break;
        }
        case 17:
        {
            if (p_multiplefocus.size()>0)
            {
                list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
                while(it!=p_multiplefocus.end())
                {

                    hk_visible* v=dynamic_cast<hk_visible*>((*it)->widget());
                    if (v)
                    {
                        v->raise_widget();
                        (*it)->widget()->raise();
                        ++it;
                    }
                }
            }

            hk_visible* v=dynamic_cast<hk_visible*>(p_focus->widget());
            if (v)
            {
                v->raise_widget();
                p_focus->widget()->raise();
                p_focus->set_widget(p_focus->widget());
            }
            break;
        }
        case 19 :   set_for_all(bulkfont);break;
        case 20:    set_for_all(bulkforeground);break;
        case 21:    set_for_all(bulkbackground);break;
        default:;
    }

    delete p_popup;
    p_popup=NULL;

}


void hk_kdesimplereport::adjust_widgets(hk_kdesimplereport::enum_adjust s)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::adjust_widgets");
#endif
    int maxw=0;
    int maxh=0;
    int minw=90000;
    int minh=90000;
    int minx=90000;
    int miny=90000;
    int maxx=0;
    int maxy=0;
    list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
    while (it!=p_multiplefocus.end())
    {
        hk_kdeformfocus* f=(*it);
        it++;
        QWidget* w=f->widget();
        if  (maxw <w->width()) maxw=w->width();
        if  (maxh <w->height()) maxh=w->height();
        if  (minw >w->width()) minw=w->width();
        if  (minh >w->height()) minh=w->height();
        if  (minx >w->x()) minx=w->x();
        if  (maxx <w->x()+w->width()) maxx=w->x()+w->width();
        if  (miny >w->y()) miny=w->y();
        if  (maxy <w->y()+w->height()) maxy=w->y()+w->height();

    }
    if (p_focus==NULL) return;
    QWidget* w=p_focus->widget();
    if (w==NULL)
    {
        return;
    }
    if  (maxw <w->width()) maxw=w->width();
    if  (maxh <w->height()) maxh=w->height();
    if  (minw >w->width()) minw=w->width();
    if  (minh >w->height()) minh=w->height();
    if  (minx >w->x()) minx=w->x();
    if  (maxx <w->x()+w->width()) maxx=w->x()+w->width();
    if  (miny >w->y()) miny=w->y();
    if  (maxy <w->y()+w->height()) maxy=w->y()+w->height();

    it=p_multiplefocus.begin();
    while (it!=p_multiplefocus.end())
    {
        hk_kdeformfocus* f=(*it);
        it++;
        resize_widget(f->widget(),s,minw,maxw,minh,maxh,minx,maxx,miny,maxy);
    }

    resize_widget(p_focus->widget(),s,minw,maxw,minh,maxh,minx,maxx,miny,maxy);
    repaint_focus();
}


void hk_kdesimplereport::resize_widget(QWidget* w,enum_adjust s,int minw,int maxw,int minh,int maxh, int minx, int maxx, int miny, int maxy)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport:resize_widget");
#endif
    switch (s)
    {
        case hk_kdesimplereport::sminw :  w->setGeometry(w->x(),w->y(),minw,w->height());
        break;
        case hk_kdesimplereport::smaxw :  w->setGeometry(w->x(),w->y(),maxw,w->height());
        break;
        case hk_kdesimplereport::sminh :  w->setGeometry(w->x(),w->y(),w->width(),minh);
        break;
        case hk_kdesimplereport::smaxh :  w->setGeometry(w->x(),w->y(),w->width(),maxh);
        break;
        case hk_kdesimplereport::smins :  w->setGeometry(w->x(),w->y(),minw,minh);
        break;
        case hk_kdesimplereport::smaxs :  w->setGeometry(w->x(),w->y(),maxw,maxh);
        break;
        case hk_kdesimplereport::al :  w->setGeometry(minx,w->y(),w->width(),w->height());
        break;
        case hk_kdesimplereport::ar :  w->setGeometry(maxx-w->width(),w->y(),w->width(),w->height());
        break;
        case hk_kdesimplereport::at :  w->setGeometry(w->x(),miny,w->width(),w->height());
        break;
        case hk_kdesimplereport::ab :  w->setGeometry(w->x(),maxy-w->height(),w->width(),w->height());
        break;
        default:;
    }

}


void hk_kdesimplereport::repaint_focus(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_kdesimplereport::repaint_focus");
#endif
    list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
    while (it!=p_multiplefocus.end())
    {
        hk_kdeformfocus* f=(*it);
        it++;
        f->set_positions();
    }
    p_focus->set_positions();
}


bool hk_kdesimplereport::is_reportobject(QWidget* v)
{
#ifdef HK_DEBUG
    //hkdebug("hk_kdesimplereport::is_reportobject");
#endif
    QWidget* object =v;

    while (object!=NULL)
    {
        if (object==this) return true;
        if (object==p_property) return false;
        object=object->parentWidget();

    }

    return false;
}


void hk_kdesimplereport::show_property(void)
{

    if (mode()!=hk_presentation::design)return;
    if (p_property==NULL)
    {
        p_property=new hk_kdereportproperty(this,0,false);
    }
    if (p_property!=NULL)  p_property->show();

}


void hk_kdesimplereport::create_copydata(ostream& stream,QWidget*w)
{
    hk_string vistag="REPORTOBJECT";
    if (w)
    {
        hk_reportdata* v = dynamic_cast<hk_reportdata*> (w);
        if (v)
        {
            start_mastertag(stream,vistag);
            v->savedata(stream,(reporttype()=="Userdefined"));
            end_mastertag(stream,vistag);
        };

    };

}


void hk_kdesimplereport::copy_widgets(void)
{
    if (mode()!=design) return;
    if (!p_focus || !p_focus->widget()) return;
    QStoredDrag* p_drag = new QStoredDrag("application/x-hk_kdesimplereportcopy");
    hk_string cpyinfo;
#ifdef HAVE_SSTREAM
    stringstream stream;
#else
    strstream stream;
#endif

    list<hk_kdeformfocus*>::iterator it=p_multiplefocus.begin();
    while (it!=p_multiplefocus.end())
    {
        create_copydata(stream,(*it)->widget());
        it++;
    }
    if (p_focus) create_copydata(stream,p_focus->widget());
    char c;
    while (stream.get(c))
    {
        cpyinfo+=c;
    }
//cout <<"cpyinfo "<<endl<<cpyinfo <<endl;

    QCString cstr =cpyinfo.c_str();
    p_drag->setEncodedData(cstr);
    QApplication::clipboard()->setData(p_drag);

}


void hk_kdesimplereport::paste_widgets(void)
{
    if (!QApplication::clipboard()->data()) return;
    if (!QApplication::clipboard()->data()->encodedData("application/x-hk_kdesimplereportcopy"))return ;
    clearfocus();
    p_controlbutton=true;
    hk_string definition=u2l(QApplication::clipboard()->data()->encodedData("application/x-hk_kdesimplereportcopy").data());
    int i=1;
    hk_string value;
    hk_string buffer2;
    hk_kdereportsection* rs=dynamic_cast<hk_kdereportsection*>((p_focussection?p_focussection:datasection()));
    if (!rs)
    {
        show_warningmessage(i18n("Error while paste action: No reportsection!!!").local8Bit().data());
        return;
    }
    while (get_tagvalue(definition,"REPORTOBJECT",value,i))
    {

        hk_reportdata* vis=rs->new_data();
        if (vis!=NULL)
        {
            vis->set_load_presentationnumber(false);
            vis->loaddata(value,(reporttype()=="Userdefined"));
            register_object(vis);
            QWidget* w=dynamic_cast<QWidget*>(vis);
            if (w)
            {
                set_focus(w,rs);
                w->repaint(true);
            }
        }
        i++;
    }

}


void hk_kdesimplereport::cut_widgets(void)
{
    copy_widgets();
    delete_widgets();

}


void hk_kdesimplereport::enable_actions(void)
{
    if (mode()==view)
    {
        if (p_copyaction)p_copyaction->setEnabled(false);
        if (p_cutaction)p_cutaction->setEnabled(false);
        if (p_pasteaction)p_pasteaction->setEnabled(false);
        if (p_deleteaction)p_deleteaction->setEnabled(false);

    }
    else
    {                                             //designmode
        if (p_copyaction)p_copyaction->setEnabled(p_focus&&p_focus->widget());
        if (p_cutaction)p_cutaction->setEnabled(p_focus&&p_focus->widget());
        if (p_deleteaction)p_deleteaction->setEnabled(p_focus&&p_focus->widget());
        if (p_pasteaction) p_pasteaction->setEnabled(QApplication::clipboard()->data()&&QApplication::clipboard()->data()->provides("application/x-hk_kdesimplereportcopy"));

    }

    if (p_resizeaction) p_resizeaction->setEnabled(mode()==design&&p_multiplefocus.size()>0);
    if (p_alignaction) p_alignaction->setEnabled(mode()==design&&p_multiplefocus.size()>0);

}


void hk_kdesimplereport::align_left(void)
{
    adjust_widgets(al);
}


void hk_kdesimplereport::align_right(void)
{
    adjust_widgets(ar);
}


void hk_kdesimplereport::align_top(void)
{
    adjust_widgets(at);
}


void hk_kdesimplereport::align_bottom(void)
{
    adjust_widgets(ab);
}


void hk_kdesimplereport::adjust_minw(void)
{
    adjust_widgets(sminw);
}


void hk_kdesimplereport::adjust_maxw(void)
{
    adjust_widgets(smaxw);
}


void hk_kdesimplereport::adjust_minh(void)
{
    adjust_widgets(sminh);
}


void hk_kdesimplereport::adjust_maxh(void)
{
    adjust_widgets(smaxh);
}


void hk_kdesimplereport::adjust_mins(void)
{
    adjust_widgets(smins);
}


void hk_kdesimplereport::adjust_maxs(void)
{
    adjust_widgets(smaxs);
}


void hk_kdesimplereport::widget_specific_presentationresize(void)
{
    set_sectionsize(datasection());
    set_sectionsize(report_header());
    set_sectionsize(report_footer());
    set_sectionsize(page_header());
    set_sectionsize(page_footer());
    vector<hk_reportsectionpair*>* sp=sectionpairs();

    if (sp)
    {
        vector<hk_reportsectionpair*>::iterator it=sp->begin();
        while (it!=sp->end())
        {
            set_sectionsize((*it)->headersection());
            set_sectionsize((*it)->footersection());
            ++it;
        }
    }

}


void hk_kdesimplereport::set_sectionsize(hk_reportsection* rs)
{
    hk_kdereportsection* s= dynamic_cast<hk_kdereportsection*>(rs);
    if (s) s->resize_section();

}


void    hk_kdesimplereport::widget_specific_backgroundcolour_changed(const hk_colour&)
{
    hk_colour c=backgroundcolour();
    QColor newcolour(c.red(),c.green(),c.blue());
    setPaletteBackgroundColor(newcolour);

}


void    hk_kdesimplereport::widget_specific_foregroundcolour_changed(const hk_colour&)
{
    hk_colour c=foregroundcolour();
    QColor newcolour(c.red(),c.green(),c.blue());
    setPaletteForegroundColor(newcolour);

}

void hk_kdesimplereport::set_for_all(enum_bulkoperation bulk)
{
    if (p_focus)
    {
        hk_visible* v= dynamic_cast<hk_visible*>(p_focus->widget());
        if (v)
        {
            switch (bulk)
            {
                case bulkforeground : set_foregroundcolour(v->foregroundcolour());
                break;
                case bulkbackground : set_backgroundcolour(v->backgroundcolour());
                break;
                case bulkfont: set_font(v->font());

            }
        }
    }
    bulk_operation(bulk);

}
